#include "buildingeditordialog.h"
#include "buildingdelegate.h"
#include "ui_buildingeditordialog.h"

#include <QMessageBox>
#include <QSqlQuery>

BuildingEditorDialog::BuildingEditorDialog(QWidget *parent) :
        QWidget(parent),
        _ui(new Ui::BuildingEditorDialog)
{
    _ui->setupUi(this);

    setupModel();
    setupFilter();
    setupUi();
}

BuildingEditorDialog::~BuildingEditorDialog()
{
    delete _ui;
    delete _model;
}

void BuildingEditorDialog::filterButton_Clicked()
{
    if (_ui->filterButton->text() == trUtf8("Szűrés")) // ha a modell nincs szűrve
    {
        _ui->nameLineEdit->setEnabled(false);
        _ui->cityComboBox->setEnabled(false);
        _ui->stateComboBox->setEnabled(false);
        _ui->filterButton->setText(trUtf8("Szűrő törlése"));

        QAbstractItemModel *currentModel = _model; // léptetjük az aktuális modellt a megszűrtek között

        if (_ui->nameLineEdit->text() != "") // ha van szöveg a szövegdobozban
        {
            _nameSortFilterModel->setSourceModel(currentModel); // forrás tábla
            _nameSortFilterModel->setFilterKeyColumn(1); // szűrés oszlopa
            _nameSortFilterModel->setFilterCaseSensitivity(Qt::CaseInsensitive); // szűrés nem függ a kis/nagybetűtől
            _nameSortFilterModel->setFilterRegExp(_ui->nameLineEdit->text()); // szűrés reguláris kifejezéssel
            currentModel = _nameSortFilterModel;
        }
        if (_ui->cityComboBox->currentText() != "mind")
        {
            _citySortFilterModel->setSourceModel(currentModel); // szűrés a városra
            _citySortFilterModel->setFilterKeyColumn(2);
            _citySortFilterModel->setFilterFixedString(_ui->cityComboBox->currentText()); // szűrés pontos illeszkedésre
            // mivel a DisplayRole-t szűrjük, ezért a legördülő menü szövege kell
            currentModel = _citySortFilterModel;
        }
        if (_ui->stateComboBox->currentText() != "mind")
        {
            _stateSortFilterModel->setSourceModel(currentModel); // szűrés az állapotra
            _stateSortFilterModel->setFilterKeyColumn(4);
            _stateSortFilterModel->setFilterFixedString(QString::number(_ui->stateComboBox->currentIndex() - 1)); // szűrés pontos illeszkedésre
            currentModel = _stateSortFilterModel;
        }

        // az ár szűrést mindenképpen alkalmazzuk
        _priceSortFilterModel->setSourceModel(currentModel);
        _priceSortFilterModel->setFilterPriceBoundaries(_ui->minPriceSpinBox->value(), _ui->maxPriceSpinBox->value());
        currentModel = _priceSortFilterModel;

        _ui->tableView->setModel(currentModel); // szűrések után kialakult modell beállítása
    }
    else
    {
        _ui->nameLineEdit->setEnabled(true);
        _ui->cityComboBox->setEnabled(true);
        _ui->stateComboBox->setEnabled(true);
        _ui->filterButton->setText(trUtf8("Szűrés"));
        _ui->tableView->setModel(_model); // szűrés nélküli modell visszaállítása
    }
}

void BuildingEditorDialog::setupModel()
{
    _model = new BuildingTableModel(this);
    _model->select();

    _nameSortFilterModel = new QSortFilterProxyModel(this); // szűrő és rendező modell
    _citySortFilterModel = new QSortFilterProxyModel(this);
    _stateSortFilterModel = new QSortFilterProxyModel(this);
    _priceSortFilterModel = new PriceSortFilterProxyModel(this);
}

void BuildingEditorDialog::setupFilter()
{
    QSqlQuery query;
    query.exec("select name from city");

    _ui->cityComboBox->addItem("mind"); // összes város felvétele
    while (query.next())
    {
        _ui->cityComboBox->addItem(query.value(0).toString()); // egy város felvétele
    }

    _ui->stateComboBox->addItem(trUtf8("mind"));
    _ui->stateComboBox->addItem(trUtf8("foglalható"));
    _ui->stateComboBox->addItem(trUtf8("felújítás alatt"));
    _ui->stateComboBox->addItem(trUtf8("nem foglalható"));
}

void BuildingEditorDialog::setupUi()
{
    _ui->tableView->setModel(_model);
    _ui->tableView->resizeColumnsToContents();
    _ui->tableView->setSelectionBehavior(QAbstractItemView::SelectRows); // teljes sor kijelölése
    _ui->tableView->setEditTriggers(QAbstractItemView::NoEditTriggers); // a táblaszerkesztés kikapcsolása a megjelenítőn
    _ui->tableView->setItemDelegate(new BuildingDelegate());
    _ui->tableView->setColumnHidden(0, true);

    connect(_ui->filterButton, SIGNAL(clicked()), this, SLOT(filterButton_Clicked()));

    setWindowTitle(trUtf8("Épületek szűrése"));
    setFixedSize(600, 400);
}
